from django.db import models
from eloqua.exceptions import ObjectNotFound
from eloqua import settings
from datetime import datetime, timedelta
import json

try:
    from django.contrib.auth import get_user_model
    user_model = django.contrib.auth.get_user_model()
except:
    # backwards compatibility for < django 1.5
    from django.contrib.auth.models import User
    user_model = User


class Contact(models.Model):
    user = models.OneToOneField(user_model, related_name='eloqua_profile')
    contact_id = models.PositiveIntegerField(null=True, blank=True)
    updated_at = models.DateTimeField(auto_now=True)
    data = models.TextField(null=True)

    _profile = None
    _contact_fields = {}

    def _fetch_profile(self):
        # check if we need to update the local data
        timeout = settings.PROFILE_TIMEOUT
        delta = timedelta(minutes=timeout)

        if self.data == None or (datetime.now() - delta) > self.updated_at:
            from eloqua.clients import EloquaClient
            e = EloquaClient()

            # fetch by eloqua id if present, otherwise,
            # search by email via the active user
            if self.contact_id == None:
                results = e.contacts.search(self.user.email)
                if len(results) > 0:
                    data = results['elements'][0]
            else:
                results = e.contacts.get(self.contact_id)
                data = results

            # json-ify the data
            self.data = json.dumps(data)
            if 'contact_id' in data:
                self.contact_id = data['content_id']
            self.save()

        # load the json data
        data = json.loads(self.data)
        self._profile = data

        # store all the contact fields seperately
        fields = {}
        for field in data['fieldValues']:
            if 'value' in field:
                fields[int(field['id'])] = field['value']
        self._contact_fields = fields

        return self._profile

    @property
    def first_name(self):
        profile = self._fetch_profile()
        return profile['firstName']

    @property
    def last_name(self):
        profile = self._fetch_profile()
        return profile['lastName']

    def value_for_field(self, pk):
        # make sure the profile is fetched
        self._fetch_profile()
        if pk in self._contact_fields:
            return self._contact_fields[pk]
        else:
            raise ObjectNotFound(pk)

# there must be a better way to do this, monkeypatching is ugly
user_model.eloqua_profile = property(lambda u: Contact.objects.get_or_create(user=u)[0])



from django.conf.urls.defaults import patterns, include, url
from .views import LandingPageView

urlpatterns = patterns('',
    url(r'^(?P<pk>[0-9]+)-?(?P<slug>[a-zA-Z0-9-_]+)?/?$', LandingPageView.as_view(),  name='eloqua:landing_page'),
)



import logging
logger = logging.getLogger(__name__)

try:
    from django.conf import settings
except ImportError:
    # We do not need Django perse
    settings = {}

# Required settings
SITE = getattr(settings, 'ELOQUA_SITE', None)
USERNAME = getattr(settings, 'ELOQUA_USERNAME', None)
PASSWORD = getattr(settings, 'ELOQUA_PASSWORD', None)

BASE_URL = getattr(settings, 'ELOQUA_BASE_URL', 'https://secure.eloqua.com/API/REST/1.0')

# how long is old data valid, and should it be locally cached (default: 24 hours)
PROFILE_TIMEOUT = getattr(settings, 'ELOQUA_PROFILE_TIMEOUT', 60 * 60 * 24)



from django.core.management.base import BaseCommand, CommandError
from datetime import date
from time import sleep
from django.conf import settings
import requests
from eloqua.clients import EloquaClient
from optparse import make_option

subjectOption = make_option('--subject',
    action='store',
    dest='subject',
    default=None,
    help='Set the subject for the mail')

nameOption = make_option('--name',
    action='store',
    dest='name',
    default=None,
    help='Set the name for this mail')

urlOption = make_option('--url',
    action='store',
    dest='url',
    default=None,
    help='Sets the url to fetch as base for this mailing')

fromNameOption = make_option('--from-name',
    action='store',
    dest='from_name',
    default=None,
    help='Sets the reply-to name for the mail')

fromEmailOption = make_option('--from-email',
    action='store',
    dest='from_email',
    default=None,
    help='Sets the reply-to email for the mail')

groupOption = make_option('--group',
    action='store',
    dest='group',
    default=None,
    help='Sets the group id for this mail')

trackOption = make_option('--track',
    action='store',
    dest='track',
    default=True,
    help='enables or disables tracking features (default: enable)')


class Command(BaseCommand):
    help = 'Create mail in Eloqua, based of a url (eg: newsletters)'
    option_list = BaseCommand.option_list + (subjectOption, nameOption, urlOption, fromEmailOption, fromNameOption, groupOption, trackOption)

    def handle(self, *args, **options):
        if 'url' not in options:
            raise CommandError('We need a URL to base from')

        try:
            response = requests.get(options['url'])
        except:
            raise CommandError('Could not fetch url')

        e = EloquaClient()
        body = response.content

        # get the subject from the title, if none given
        if options['subject'] is None:
            from BeautifulSoup import BeautifulSoup
            soup = BeautifulSoup(body)
            options['subject'] = soup.title.string

        # if no name is given, set the name the same as the subject
        if options['name'] is None:
            options['name'] = options['subject']

        import pprint
        pp = pprint.PrettyPrinter(indent=4)
        pp.pprint(e.emails.create(options['name'], options['subject'], body, group_id=options['group'], sender_email=options['from_email'], sender_name=options['from_name'], is_tracked=options['track']))



#!/usr/bin/env python
from setuptools import setup, find_packages

try:
    README = open('README.rst').read()
except:
    README = None

try: 
    LICENSE = open('LICENSE.txt').read()
except: 
    LICENSE = None

setup(
    name = 'django-eloqua',
    version = '0.4.0',
    description='Python/Django interface for the Eloqua REST api.',
    long_description=README,
    author = 'Sander van de Graaf',
    author_email = 'mail@svdgraaf.nl',
    license = LICENSE,
    url = 'http://github.com/svdgraaf/django-eloqua/',
    packages = find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Framework :: Django',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Environment :: Web Environment',
    ],
    install_requires=[
        "Django >= 1.3.0",
        "requests >= 1.1.0",

    ],
)



from eloqua import settings
from eloqua.exceptions import ObjectNotFound
import requests
import base64
import json


class EloquaBaseClient(object):
    headers = ''
    base_url = settings.BASE_URL

    def __init__(self):
        key = '{site}\\{username}:{password}'.format(site=settings.SITE, username=settings.USERNAME, password=settings.PASSWORD)
        authKey = base64.b64encode(key)
        self.headers = {"Content-Type": "application/json", "Authorization": "Basic " + authKey}

    def get(self, identifier):
        """Return the asset by the given id"""
        return self._get_by_id(identifier)

    def _get_by_id(self, identifier):
        url = self.base_url + '/' + str(identifier)
        print url
        response = requests.get(url, headers=self.headers)
        if response.status_code == 200:
            return response.json()
        else:
            raise ObjectNotFound(identifier)

    # Delete asset by id, return True or False when succesful
    def _delete_by_id(self, identifier):
        url = self.base_url + '/' + str(identifier)
        response = requests.delete(url, headers=self.headers)
        if response.status_code == 200:
            return True
        else:
            return False

    def delete(self, identifier):
        """Deletes the asset by the given id"""
        return self._delete_by_id(identifier)

    def search(self, query, page=1, count=100, depth='complete'):
        url = self.base_url + 's'
        payload = {
            'search': query,
            'page': page,
            'count': count,
            'depth': depth,
        }

        r = requests.get(url, params=payload, headers=self.headers)
        return r.json()


class EloquaClient(EloquaBaseClient):
    headers = ''
    base_url = settings.BASE_URL

    @property
    def emails(self):
        return EloquaEmailClient()

    @property
    def email_groups(self):
        return EloquaEmailGroupsClient()

    @property
    def contacts(self):
        return EloquaContactsClient()

    @property
    def contact_fields(self):
        return EloquaContactFieldsClient()

    @property
    def landingpages(self):
        return EloquaLandingPagesClient()


class EloquaEmailClient(EloquaBaseClient):
    # only get/search need the slash
    base_url = settings.BASE_URL + '/assets/email'

    def create(self, name, subject, body, body_plaintext=None, group_id=None, is_tracked=False, folder_id=None, sender_name=None, sender_email=None, reply_to_email=None, reply_to_name=None):
        """Creates an email with the given parameters, returns a json representation"""
        payload = {
            'htmlContent': {
                'html': body,
                'type': 'RawHtmlContent',
            },
            'sendPlainTextOnly': False,
            'name': name,
            'subject': subject,
            'id': None,
            'isTracked': is_tracked,
        }

        if folder_id:
            payload['folderId'] = folder_id

        # only add, if provided, otherwise, let the system generate one
        if body_plaintext:
            payload['body_plaintext'] = body_plaintext

        if group_id:
            payload['emailGroupId'] = group_id

        if sender_email:
            payload['senderEmail'] = sender_email
        if sender_name:
            payload['senderName'] = sender_name

        if reply_to_email:
            payload['replyToEmail'] = reply_to_email
        if reply_to_name:
            payload['replyToName'] = reply_to_name

        # make the call
        # TODO: logging?
        r = requests.post(self.base_url, data=json.dumps(payload), headers=self.headers)
        if r.status_code == 201:
            return r.json()
        else:
            raise Exception('error: %s' % r)  # TODO: be more verbose


class EloquaContactsClient(EloquaBaseClient):
    # only get/search need the slash
    base_url = settings.BASE_URL + '/data/contact'

    def search(self, query, page=1, count=100, depth='complete'):
        """Search all contacts for given email query, eg:

            from eloqua.lib import *
            e = EloquaClient()
            e.contacts.search('foobar@example.com')
                {u'elements': [{u'createdAt': u'1327661283',
                u'currentStatus': u'Awaiting action',
                u'depth': u'complete',
                u'emailAddress': u'foobar@example.com',
                ...

        """
        url = self.base_url + 's'
        payload = {
            'search': query,
            'page': page,
            'count': count,
            'depth': depth,
        }

        r = requests.get(url, params=payload, headers=self.headers)
        if r.status_code == 200:
            return r.json()
        else:
            raise ObjectNotFound(None)


class EloquaContactFieldsClient(EloquaBaseClient):
    base_url = settings.BASE_URL + '/assets/contact/field'

    def search(self, query, page=1, count=100, depth='complete'):
        url = self.base_url + 's'
        payload = {
            'search': query,
            'page': page,
            'count': count,
            'depth': depth,
        }

        r = requests.get(url, params=payload, headers=self.headers)
        return r.json()


class EloquaEmailGroupsClient(EloquaBaseClient):
    base_url = settings.BASE_URL + '/assets/email/group'

    def search(self, query, page=1, count=100, depth='complete'):
        url = self.base_url + 's'
        payload = {
            'search': query,
            'page': page,
            'count': count,
            'depth': depth,
        }

        r = requests.get(url, params=payload, headers=self.headers)
        return r.json()


class EloquaLandingPagesClient(EloquaBaseClient):
    base_url = settings.BASE_URL + '/assets/landingPage'



from django.views.generic.base import TemplateView
from .clients import EloquaLandingPagesClient


class LandingPageView(TemplateView):
    template_name = "eloqua/landing_page.html"

    def get_context_data(self, **kwargs):
        context = super(LandingPageView, self).get_context_data(**kwargs)

        # TODO: add try/catch if page not found?
        e = EloquaLandingPagesClient()
        landing_page = e.get(kwargs['pk'])

        context['landing_page'] = landing_page

        return context
